package com.afp.ordermanagement.controller;

import com.afp.ordermanagement.exception.BadAccessTokenException;
import com.afp.ordermanagement.exception.EntityNotFoundException;
import com.afp.ordermanagement.model.Manager;
import com.afp.ordermanagement.service.ManagerService;
import com.afp.ordermanagement.service.ManagerTokenVerifier;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import javax.validation.Valid;

@Validated
@RestController
@RequestMapping("/api/managers/")
public class ManagerController {
    @Autowired
    ManagerService managerService;

    @GetMapping("/")
    public Flux<Manager> getAllManagers() {
        return managerService.getAll();
    }

    @PostMapping("/")
    @ResponseStatus(HttpStatus.CREATED)
    public Mono<Manager> createManager(@Valid @RequestBody Manager newManager) {
        return managerService.create(newManager);
    }

    @PostMapping("/auth")
    public Mono<Manager> getOrCreateManagerFromAccessToken(@RequestHeader(name = "Authorization") String tokenHeader) {
        String tokenStr = tokenHeader.replace("Bearer ", "");
        if (!ManagerTokenVerifier.isTokenValid(tokenStr))
            return Mono.error(new BadAccessTokenException());
        Manager managerPayload = ManagerTokenVerifier.createManager(tokenStr);
        return managerService.getByEmail(managerPayload.getEmail())
                .switchIfEmpty(managerService.create(managerPayload));
    }


    @GetMapping("/{uuid}")
    public Mono<Manager> getManagerById(@PathVariable String uuid) {
        return managerService.getById(uuid)
                .switchIfEmpty(Mono.error(new EntityNotFoundException(uuid, Manager.class)));
    }

    @PatchMapping("/{uuid}")
    public Mono<Manager> updateManagerById(@PathVariable String uuid, @RequestBody @Valid Manager managerUpdates) {
        return managerService.updateById(uuid, managerUpdates);
    }

    @DeleteMapping("/{uuid}")
    @ResponseStatus(HttpStatus.NO_CONTENT)
    public Mono<Void> deleteManagerById(@PathVariable String uuid) {
        return managerService.getById(uuid)
                .switchIfEmpty(Mono.error(new EntityNotFoundException(uuid, Manager.class)))
                .flatMap(managerService::delete);
    }
}
